{==================================================================

 Basic file handling utilities  /  Version 1.0  / 9-2000
 ========================================================


    Description:
    ============
    Basic utilities useful for handling files or directories.
    Used by the components TDriveView and TDirView.

    Author:
    =======
    (c) Hubert Indetzki 9-2000
    Dammer Hof 12
    49088 Osnabrueck
    Germany

{==================================================================}



unit BasicUnit;


interface

Uses SysUtils, Windows;


function RemoveSlash ( s : string ) : string;
function AddSlash ( s : string ) : string;
function ExtractLastFolder ( path : string) : string;
function LastPos ( s : string; ch : char ) : integer;
function DiskInDrive ( drive: char ) : boolean;
function FileOrDirExists ( path : string ) : boolean;
function FormatSize ( size : integer ) : string;
function StrDeleteChar(S : String; Ch : Char) : String;


implementation




function LastPos ( s : string; ch : char ) : integer;
var
  i : integer;
begin
  result := 0;
  for i := length(s) downto 1 do
   if s[i] = ch then
    begin
     result := i;
     exit;
    end;
end;


function ExtractLastFolder ( path : string) : string;
var
  pos : integer;
begin
  path := RemoveSlash ( path );
  pos   := LastPos ( path, '\' );
  if pos = 0 then
   result := ''
    else
     result := copy ( path, succ(pos), length(path) - pos);
end;


function RemoveSlash ( s : string ) : string;
begin
  if (length(s) > 0) and (s[length(s)] = '\') then
   result := copy ( s, 1, pred(length(s)) )
    else
     result := s;
end;


function AddSlash ( s : string ) : string;
begin
  if ( length(s) > 0 ) and ( s[length(s)] <> '\' ) then
   result := s + '\'
    else
     result := s;
end;


function FileOrDirExists ( path : string ) : boolean;
Var
  f : tSearchRec;
begin
  result := FindFirst ( AddSlash(path) + '*.*', faAnyFile, f ) = 0;
  SysUtils.FindCLose ( f );
end;



function DiskInDrive ( drive: char ) : boolean;
var
  ErrorMode : word;
begin
  drive := UpCase(drive);
  if not ( drive in ['A'..'Z']) then
    raise EConvertError.create ( 'Not a valid drive ID' );
  ErrorMode := SetErrorMode ( SEM_FailCriticalErrors or Sem_NoOpenFileErrorBox );
  try
   if DiskSize(ord(drive) - $40) = -1 then
    result := false
    else
      result := true;
  finally    { restore old error mode }
    SetErrorMode ( ErrorMode );
  end;
end;



function FormatSize ( size : integer ) : string;
var
  i : integer;
  p : integer;
begin
  i := 3;
  p := 0;
  result := IntToStr ( size );
  while i + p < length(result) do
   begin
    insert ( '.', result, length(result) - (i + p) + 1 );
    inc ( i, 3 );
    inc ( p );
   end;
end;



function StrDeleteChar(S : String; Ch : Char) : String;
Var chPos : Integer;
Begin
  chPos := Pos(Ch, S);
  WHile chPos > 0 Do
  Begin
    Delete(S, chPos, 1);
    chPos := Pos(Ch, S);
  End;
  Result := S;
End; {StrDelChar}


END.
