unit deBoldAutoIncManager;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: deBoldAutoIncManager.PAS, released December 4, 2002.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2002 Peter Morris.
All Rights Reserved.

Purpose of file:
Adds the ability for auto-increment values to be assigned independantly of the
underlying database. 

Contributor(s):
<None>

Last Modified: December 9, 2002.

You may retrieve the latest version of this file at
located at http://www.droopyeyes.com

NOTE!!
To make an attribute 
-----------------------------------------------------------------------------}

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, BoldPersistenceHandlePassthrough, BoldPersistenceController,
  BoldSystemHandle, BoldID, BoldValueSpaceInterfaces, BoldDefs, BoldCondition,
  BoldUpdatePrecondition, BoldAbstractSnooper, BoldSystem;

type
  TdeCustomBoldAutoIncManager = class;
  TdeAbstractBoldAutoIncProvider = class;
  TAutoIncIdListEvent = procedure (Sender: TObject; const AutoIncIdList: 
          TStrings) of object;
  TGetAutoIncValueEvent = procedure (Sender: TObject; AutoIncId: string; var 
          Result: Integer) of object;
  EBoldAutoIncManagerError = class (Exception)
  end;
  
  TdeBoldAutoIncManagerPersistenceController = class (TBoldAbstractSnooper)
  private
    FAutoIncManager: TdeCustomBoldAutoIncManager;
    procedure ManagerDestroying;
  public
    destructor Destroy; override;
    procedure InheritedPMUpdate(ObjectIdList: TBoldObjectIdList; ValueSpace: 
            IBoldValueSpace; Old_Values: IBoldValueSpace; Precondition: 
            TBoldUpdatePrecondition; TranslationList: TBoldIdTranslationList; 
            var TimeStamp: TBoldTimeStampType; BoldClientID: TBoldClientID);
    procedure PMUpdate(ObjectIdList: TBoldObjectIdList; ValueSpace: 
            IBoldValueSpace; Old_Values: IBoldValueSpace; Precondition: 
            TBoldUpdatePrecondition; TranslationList: TBoldIdTranslationList; 
            var TimeStamp: TBoldTimeStampType; BoldClientID: TBoldClientID); 
            override;
    procedure TransmitEvents(const ClientID: TBoldClientID); override;
  end;
  
  TdeCustomBoldAutoIncManager = class (TBoldPersistenceHandlePassthrough)
  private
    FBoldAutoIncProvider: TdeAbstractBoldAutoIncProvider;
    FControllerList: TList;
    FObjectIdList: TBoldObjectIdList;
    FSystemHandle: TBoldSystemHandle;
    procedure ControllerDestroying(Controller: 
            TdeBoldAutoIncManagerPersistenceController);
    procedure SetSystemHandle(Value: TBoldSystemHandle);
  protected
    function CreatePersistenceController: TBoldPersistenceController; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); 
            override;
    procedure PMUpdate(Sender: TdeBoldAutoIncManagerPersistenceController; 
            ObjectIdList: TBoldObjectIdList; ValueSpace: IBoldValueSpace; 
            Old_Values: IBoldValueSpace; Precondition: TBoldUpdatePrecondition; 
            TranslationList: TBoldIdTranslationList; var TimeStamp: 
            TBoldTimeStampType; BoldClientID: TBoldClientID); virtual;
    procedure SetActive(Value: Boolean); override;
    procedure SetBoldAutoIncProvider(Value: TdeAbstractBoldAutoIncProvider); 
            virtual;
    property BoldAutoIncProvider: TdeAbstractBoldAutoIncProvider read 
            FBoldAutoIncProvider write SetBoldAutoIncProvider;
    property SystemHandle: TBoldSystemHandle read FSystemHandle write 
            SetSystemHandle;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    procedure AddDirtyObject(BoldObject: TBoldObject);
    procedure AddDirtyObjectList(BoldObjectList: TBoldObjectList);
    procedure AddDirtyObjects(Objects: array of TBoldObject);
  end;
  
  TdeBoldAutoIncManager = class (TdeCustomBoldAutoIncManager)
  published
    property BoldAutoIncProvider;
    property NextPersistenceHandle;
    property SystemHandle;
  end;
  
  TdeAbstractBoldAutoIncProvider = class (TComponent)
  private
    FAutoIncManager: TdeCustomBoldAutoIncManager;
    FResetOnFail: Boolean;
  protected
    procedure CompleteAutoIncIdList(const AutoIncIdList: TStrings); virtual; 
            abstract;
    procedure GetAutoIncValue(AutoIncId: string; var Result: Integer); virtual; 
            abstract;
    procedure PrepareAutoIncIdList(const AutoIncIdList: TStrings); virtual; 
            abstract;
    property ResetOnFail: Boolean read FResetOnFail write FResetOnFail;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddDirtyObject(BoldObject: TBoldObject);
    procedure AddDirtyObjectList(BoldObjectList: TBoldObjectList);
    procedure AddDirtyObjects(Objects: array of TBoldObject);
  end;
  
  TdeCustomAutoIncProvider = class (TdeAbstractBoldAutoIncProvider)
  private
    FOnCompleteAutoIncIdList: TAutoIncIdListEvent;
    FOnGetAutoIncValue: TGetAutoIncValueEvent;
    FOnPrepareAutoIncIdList: TAutoIncIdListEvent;
  protected
    procedure CompleteAutoIncIdList(const AutoIncIdList: TStrings); override;
    procedure GetAutoIncValue(AutoIncId: string; var Result: Integer); override;
    procedure PrepareAutoIncIdList(const AutoIncIdList: TStrings); override;
    property OnCompleteAutoIncIdList: TAutoIncIdListEvent read 
            FOnCompleteAutoIncIdList write FOnCompleteAutoIncIdList;
    property OnGetAutoIncValue: TGetAutoIncValueEvent read FOnGetAutoIncValue 
            write FOnGetAutoIncValue;
    property OnPrepareAutoIncIdList: TAutoIncIdListEvent read 
            FOnPrepareAutoIncIdList write FOnPrepareAutoIncIdList;
  end;
  
  TdeBoldAutoIncProvider = class (TdeCustomAutoIncProvider)
  published
    property OnCompleteAutoIncIdList;
    property OnGetAutoIncValue;
    property OnPrepareAutoIncIdList;
    property ResetOnFail default True;
  end;
  

implementation
uses
  BoldMeta, BoldValueInterfaces, Contnrs, BoldPersistenceHandle;

{
****************** TdeBoldAutoIncManagerPersistenceController ******************
}
destructor TdeBoldAutoIncManagerPersistenceController.Destroy;
begin
  if Assigned(FAutoIncManager) then
  begin
    FAutoIncManager.ControllerDestroying(Self);
    FAutoIncManager := nil;
  end;
  inherited Destroy;
end;

procedure TdeBoldAutoIncManagerPersistenceController.InheritedPMUpdate(
        ObjectIdList: TBoldObjectIdList; ValueSpace: IBoldValueSpace; 
        Old_Values: IBoldValueSpace; Precondition: TBoldUpdatePrecondition; 
        TranslationList: TBoldIdTranslationList; var TimeStamp: 
        TBoldTimeStampType; BoldClientID: TBoldClientID);
begin
  inherited PMUpdate(ObjectIdList, ValueSpace, Old_Values, Precondition,
    TranslationList, TimeStamp, BoldClientID);
end;

procedure TdeBoldAutoIncManagerPersistenceController.ManagerDestroying;
begin
  FAutoIncManager.ControllerDestroying(Self);
  FAutoIncManager := nil;
end;

procedure TdeBoldAutoIncManagerPersistenceController.PMUpdate(ObjectIdList: 
        TBoldObjectIdList; ValueSpace: IBoldValueSpace; Old_Values: 
        IBoldValueSpace; Precondition: TBoldUpdatePrecondition; 
        TranslationList: TBoldIdTranslationList; var TimeStamp: 
        TBoldTimeStampType; BoldClientID: TBoldClientID);
begin
  FAutoIncManager.PMUpdate(Self, ObjectIdList, ValueSpace, Old_Values,
    Precondition, TranslationList, TimeStamp, BoldClientId);
end;

procedure TdeBoldAutoIncManagerPersistenceController.TransmitEvents(const 
        ClientID: TBoldClientID);
begin
end;

{
************************* TdeCustomBoldAutoIncManager **************************
}
constructor TdeCustomBoldAutoIncManager.Create(Owner: TComponent);
begin
  inherited;
  FControllerList := TList.Create;
end;

destructor TdeCustomBoldAutoIncManager.Destroy;
var
  I: Integer;
begin
  for I := FControllerList.Count - 1 downto 0 do
    TdeBoldAutoIncManagerPersistenceController(FControllerList[I]).ManagerDestroying;
  inherited Destroy;
end;

procedure TdeCustomBoldAutoIncManager.AddDirtyObject(BoldObject: TBoldObject);
begin
  FObjectIdList.Add(BoldObject.BoldObjectLocator.BoldObjectID);
end;

procedure TdeCustomBoldAutoIncManager.AddDirtyObjectList(BoldObjectList: 
        TBoldObjectList);
var
  I: Integer;
begin
  for I := 0 to BoldObjectList.Count - 1 do
    AddDirtyObject(BoldObjectList[I]);
end;

procedure TdeCustomBoldAutoIncManager.AddDirtyObjects(Objects: array of 
        TBoldObject);
var
  I: Integer;
begin
  for I := Low(Objects) to High(Objects) do
    AddDirtyObject(Objects[I]);
end;

procedure TdeCustomBoldAutoIncManager.ControllerDestroying(Controller: 
        TdeBoldAutoIncManagerPersistenceController);
begin
  FControllerList.Remove(Controller);
end;

function TdeCustomBoldAutoIncManager.CreatePersistenceController: 
        TBoldPersistenceController;
begin
  Result := TdeBoldAutoIncManagerPersistenceController.Create(SystemHandle.SystemTypeInfoHandle.BoldModel.MoldModel);
  with TdeBoldAutoIncManagerPersistenceController(Result) do
  begin
    NextPersistenceController := Self.NextPersistenceHandle.PersistenceController;
    FAutoIncManager := Self;
  end;
  FControllerList.Add(Result);
end;

procedure TdeCustomBoldAutoIncManager.Loaded;
begin
  BoldAutoIncProvider := BoldAutoIncProvider;
end;

procedure TdeCustomBoldAutoIncManager.Notification(AComponent: TComponent; 
        Operation: TOperation);
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = NextPersistenceHandle then
      NextPersistenceHandle := nil;
    if AComponent = BoldAutoIncProvider then
      BoldAutoIncProvider := nil;
    if AComponent = SystemHandle then
      SystemHandle := nil;
  end;
end;

procedure TdeCustomBoldAutoIncManager.PMUpdate(Sender: 
        TdeBoldAutoIncManagerPersistenceController; ObjectIdList: 
        TBoldObjectIdList; ValueSpace: IBoldValueSpace; Old_Values: 
        IBoldValueSpace; Precondition: TBoldUpdatePrecondition; 
        TranslationList: TBoldIdTranslationList; var TimeStamp: 
        TBoldTimeStampType; BoldClientID: TBoldClientID);
var
  I: Integer;
  J: Integer;
  AClass: TMoldClass;
  Element: TMoldElement;
  ObjectContents: IBoldObjectContents;
  AutoIncId: string;
  Value: IBoldValue;
  IntegerValue: IBoldIntegerContent;
  AutoIncAttributeList: TInterfaceList;
  AutoIncIdList: TStringList;
  UniqueAutoIncIdList: TStringList;
  AutoIncValue: Integer;
  MoldModel: TMoldModel;
begin
  if not Assigned(BoldAutoIncProvider) then
    raise EBoldAutoIncManagerError.Create('No auto-inc provider specified.');
  if not Assigned(SystemHandle) then
    raise EBoldAutoIncManagerError.Create('AutoIncManager has no SystemHandle.');
  
  FObjectIdList := ObjectIdList;
  BoldAutoIncProvider.FAutoIncManager := Self;
  AutoIncAttributeList := TInterfaceList.Create;
  AutoIncIdList := TStringList.Create;
  UniqueAutoIncIdList := TStringList.Create;
  MoldModel := SystemHandle.SystemTypeInfoHandle.BoldModel.MoldModel;
  try
    UniqueAutoIncIdList.Sorted := True;
    for I := 0 to ObjectIdList.Count - 1 do
    begin
      ObjectContents := ValueSpace.ObjectContentsByObjectId[ObjectIdList[I]];
      if (ObjectContents.BoldExistenceState = besExisting) and
        (ObjectContents.BoldPersistenceState = bvpsModified) then
      begin
        //New object!
        AClass := MoldModel.Classes.Items[ObjectIdList[i].TopSortedIndex];
        for J := 0 to ObjectContents.MemberCount - 1 do
        begin
          Element := AClass.AllBoldMembers[J];
          AutoIncId := Element.TVByName['deBold.AutoIncId'];
          if AutoIncId = '' then Continue;
          Value := ObjectContents.ValueByIndex[J];
          if not Assigned(Value) then Continue;
          if Value.QueryInterface(IBoldIntegerContent, IntegerValue) <> S_OK then Continue;
          if IntegerValue.AsInteger = 0 then
          begin
            AutoIncAttributeList.Add(IntegerValue);
            AutoIncIdList.Add(AutoIncId);
            if UniqueAutoIncIdList.IndexOf(AutoIncId) = -1 then
              UniqueAutoIncIdList.Add(AutoIncId);
          end;
        end; //for J
      end; //if new object
    end;  //for I
  
    try
      try
        //Prepare the AutoIncId list (Pessimistic locking, or select the objects from PS
        if AutoIncAttributeList.Count > 0 then
        begin
          BoldAutoIncProvider.PrepareAutoIncIdList(UniqueAutoIncIdList);
  
          //Assign the auto-inc numbers
          for I := 0 to AutoIncAttributeList.Count - 1 do
          begin
            AutoIncValue := -1;
            BoldAutoIncProvider.GetAutoIncValue(AutoIncIdList[I], AutoIncValue);
            if AutoIncValue = -1 then
              raise EBoldAutoIncManagerError.Create('AutoIncProvider did not provide an ' +
                'auto inc value for AutoIncId: ' + AutoIncIdList[I]);
            (AutoIncAttributeList[I] as IBoldIntegerContent).AsInteger := AutoIncValue;
          end;
        end;
  
        //Update the PS
        Sender.InheritedPMUpdate(ObjectIdList, ValueSpace, Old_Values, Precondition,
          TranslationList, TimeStamp, BoldClientId);
      except
        if BoldAutoIncProvider.ResetOnFail then
          for I := 0 to AutoIncAttributeList.Count - 1 do
            (AutoIncAttributeList[I] as IBoldIntegerContent).AsInteger := 0;
      end;
    finally
      if AutoIncAttributeList.Count > 0 then
        BoldAutoIncProvider.CompleteAutoIncIdList(UniqueAutoIncIdList);
    end;
  finally
    FObjectIdList := nil;
    AutoIncIdList.Free;
    AutoIncAttributeList.Free;
    UniqueAutoIncIdList.Free;
  end;
end;

procedure TdeCustomBoldAutoIncManager.SetActive(Value: Boolean);
begin
  if not Assigned(SystemHandle) then
    raise EBoldAutoIncManagerError.Create('No SystemHandle specified.');
  inherited SetActive(Value);
end;

procedure TdeCustomBoldAutoIncManager.SetBoldAutoIncProvider(Value: 
        TdeAbstractBoldAutoIncProvider);
begin
  FBoldAutoIncProvider := Value;
  if Assigned(BoldAutoIncProvider) then
    BoldAutoIncProvider.FreeNotification(Self);
end;

procedure TdeCustomBoldAutoIncManager.SetSystemHandle(Value: TBoldSystemHandle);
var
  NextPersistence: TBoldPersistenceHandle;
begin
  if Value = FSystemHandle then Exit;
  FSystemHandle := Value;
  if Assigned(SystemHandle) then
  begin
    NextPersistence := NextPersistenceHandle;
    SystemHandle.FreeNotification(Self);
    NextPersistenceHandle := nil;
    if Assigned(NextPersistence) then
      NextPersistenceHandle := NextPersistence;
  end;
end;

{
************************ TdeAbstractBoldAutoIncProvider ************************
}
constructor TdeAbstractBoldAutoIncProvider.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FResetOnFail := True;
end;

procedure TdeAbstractBoldAutoIncProvider.AddDirtyObject(BoldObject: 
        TBoldObject);
begin
  FAutoIncManager.AddDirtyObject(BoldObject);
end;

procedure TdeAbstractBoldAutoIncProvider.AddDirtyObjectList(BoldObjectList: 
        TBoldObjectList);
begin
  FAutoIncManager.AddDirtyObjectList(BoldObjectList);
end;

procedure TdeAbstractBoldAutoIncProvider.AddDirtyObjects(Objects: array of 
        TBoldObject);
begin
  FAutoIncManager.AddDirtyObjects(Objects);
end;

{
*************************** TdeCustomAutoIncProvider ***************************
}
procedure TdeCustomAutoIncProvider.CompleteAutoIncIdList(const AutoIncIdList: 
        TStrings);
begin
  if Assigned(FOnCompleteAutoIncIdList) then
    FOnCompleteAutoIncIdList(Self, AutoIncIdList);
end;

procedure TdeCustomAutoIncProvider.GetAutoIncValue(AutoIncId: string; var 
        Result: Integer);
begin
  if Assigned(FOnGetAutoIncValue) then
    FOnGetAutoIncValue(Self, AutoIncId, Result);
end;

procedure TdeCustomAutoIncProvider.PrepareAutoIncIdList(const AutoIncIdList: 
        TStrings);
begin
  if Assigned(FOnPrepareAutoIncIdList) then
    FOnPrepareAutoIncIdList(Self, AutoIncIdList);
end;


end.
